#ifndef __CUI_H__
#define __CUI_H__

#define CUI_DRAW_NORMAL		0
#define CUI_DRAW_FOCUS		1
#define CUI_DRAW_ACTIVE		2

typedef struct cui_base *cui;

#define MSG(s) fprintf(stderr, "%s() %s L%d : %s\n", __func__, __FILE__, __LINE__, s)
#define ERR(s) do{ fprintf(stderr, "ERR "); MSG(s); exit(1); }while(0)

#define FALSE	0
#define TRUE	1

#define CUI_EVT_KEY		(1<<0)
#define CUI_EVT_DRAW		(1<<1)
#define CUI_EVT_BUTTON		(1<<2)

#define CUI_FLG_CAN_FOCUS	(1<<0)
#define CUI_FLG_HIDE		(1<<1)

#define CUI_ATTR_NORMAL		0
#define CUI_ATTR_ULINE		(1<<0)
#define CUI_ATTR_REVERSE	(1<<1)

typedef struct cui_handler_list *cui_handler_list;

#include "rect.h"

struct cui_base{
	cui parent, children, next;
	int x, y, w, h;
	cui_handler_list handler_list;
	int flags;
};	

cui cui_alloc(int size);
void cui_base_init(cui obj, cui parent, int x, int y, int w, int h);
void cui_base_child_add(cui obj, cui child);
cui cui_base_new(cui parent, int x, int y, int w, int h);

void cui_draw_str(cui obj, int x, int y, char *s, int attr);
void cui_fill_rect(cui obj, int x, int y, int w, int h, char *s, int attr);
void cui_clear_rect(cui obj, int x, int y, int w, int h);
void cui_clear(cui obj);

int cui_gx(cui obj);
int cui_gy(cui obj);
int cui_glen(cui o1, cui o2);
void cui_draw(cui obj);
void cui_draw_chain(cui obj);
void cui_hide(cui obj);
void cui_show(cui obj);
int cui_is_visible(cui obj);
int cui_visible_rect(cui obj, cui_rect r);
int cui_is_visible_rect(cui obj);
void cui_xywh_set(cui obj, int x, int y, int w, int h);
void cui_x_set(cui obj, int x);
void cui_w_set(cui obj, int w);

cui cui_focus_get(void);
void cui_focus_set(cui obj);
void cui_focus_chk(void);
int cui_running_get(void);
void cui_running_set(int v);

void cui_quit(void);
void cui_main(cui top_obj, cui init_focus);
void cui_del(cui obj);
void cui_free_chain(cui obj);
void cui_free(cui obj);

#endif
